@@ -40,7 +40,8 @@
 #include <Lum/Model/Header.h>
 #include <Lum/Model/String.h>
 
-#include <Configuration.h>
+#include "Configuration.h"
+#include "Trend.h"
 #include "Util.h"
 
 #include <iostream>
@@ -69,7 +70,6 @@ public:
     string=new Lum::String();
     string->SetFlex(false,false);
     string->SetMinWidth(Lum::Base::Size::stdCharWidth,6);
-    string->RequestFocus();
 
     tmp=_(L"WEIGHT_EDIT_LABEL_WEIGHT",L"Weight ($unit):");
     Lum::Base::L10N::Substitute(tmp,L"$unit",weightUnit->Get());
@@ -122,7 +122,6 @@ public:
     std::wstring tmp;
 
     string=new Lum::String();
-    string->RequestFocus();
     string->SetMinWidth(Lum::Base::Size::stdCharWidth,20);
 
     v.label=_(L"FOOD_EDIT_LABEL_NAME",L"Name:");
@@ -132,7 +131,6 @@ public:
     AddValue(v);
 
     string=new Lum::String();
-    string->RequestFocus();
     string->SetMinWidth(Lum::Base::Size::stdCharWidth,6);
 
     v.label=_(L"FOOD_EDIT_LABEL_UNIT",L"Unit:");
@@ -143,7 +141,6 @@ public:
 
     string=new Lum::String();
     string->SetMinWidth(Lum::Base::Size::stdCharWidth,4);
-    string->RequestFocus();
 
     tmp=_(L"FOOD_EDIT_LABEL_VALUE",L"Value ($unit):");
     Lum::Base::L10N::Substitute(tmp,L"$unit",valueUnit->Get());
@@ -208,7 +205,6 @@ public:
 
     string=new Lum::String();
     string->SetFlex(false,false);
-    string->RequestFocus();
     string->SetMinWidth(Lum::Base::Size::stdCharWidth,4);
 
     tmp=_(L"FOOD_REGISTER_LABEL_AMOUNT",L"Amount ($unit):");
@@ -355,7 +351,7 @@ public:
      badFoodColor(0.95,0.85,0.85,Lum::OS::display->GetColor(Lum::OS::Display::whiteColor)),
      goodFoodBackground(new Lum::OS::PlainFill(goodFoodColor)),
      badFoodBackground(new Lum::OS::PlainFill(badFoodColor))
-   {
+  {
     // no code
   }
 
@@ -411,7 +407,8 @@ DataInput::DataInput()
    foodSelection(new Lum::Model::SingleLineSelection()),
    entrySelection(new Lum::Model::SingleLineSelection()),
 
-   searchTimerAction(new Lum::Model::Action())
+   searchTimerAction(new Lum::Model::Action()),
+   trend(new Lum::Model::Int(Trend::none))
 {
   SetPrefs(::prefs);
 
@@ -438,10 +435,18 @@ DataInput::DataInput()
 
   AttachModel(goodValueLimit);
   AttachModel(badValueLimit);
+
+  AttachModel(trendType);
+  AttachModel(trendDirection);
+  AttachModel(refValue);
 }
 
 DataInput::~DataInput()
 {
+  UnattachModel(refValue);
+  UnattachModel(trendDirection);
+  UnattachModel(trendType);
+
   UnattachModel(goodValueLimit);
   UnattachModel(badValueLimit);
 
@@ -479,6 +484,7 @@ void DataInput::CalcSize()
   Lum::Table            *table;
   Lum::TextValue        *text;
   Lum::Model::HeaderRef headerModel;
+  Trend                 *trend;
 
   // Search box
 
@@ -487,7 +493,6 @@ void DataInput::CalcSize()
 
   string=new Lum::String();
   string->SetFlex(true,false);
-  string->RequestFocus();
   string->SetModel(foodSearch);
   sb->Add(string);
 
@@ -498,12 +503,11 @@ void DataInput::CalcSize()
 
   headerModel=new Lum::Model::HeaderImpl();
   headerModel->AddColumn(_(L"FOOD_TABLE_HEADER_FOOD",L"Food"),Lum::Base::Size::stdCharWidth,15,true);
-  headerModel->AddColumn(_(L"FOOD_TABLE_HEADER_VALUE",L"Value"),Lum::Base::Size::stdCharWidth,6);
+  headerModel->AddColumn(_(L"FOOD_TABLE_HEADER_VALUE",L"Value"),Lum::Base::Size::stdCharWidth,8);
 
   table=new Lum::Table();
   table->SetFlex(true,true);
   table->SetMinWidth(Lum::Base::Size::stdCharWidth,25);
-  table->RequestFocus();
   table->SetHeaderModel(headerModel);
   table->SetModel(foods);
   table->SetSelection(foodSelection);
@@ -527,7 +531,6 @@ void DataInput::CalcSize()
   ds->SetFlex(true,false);
 
   button=new Lum::Button(L"_<");
-  button->RequestFocus();
   button->SetModel(prevDayAction);
   ds->Add(button);
 
@@ -540,16 +543,21 @@ void DataInput::CalcSize()
   ds->Add(new Lum::HSpace(Lum::Space::sizeSmall));
 
   button=new Lum::Button(L"_>");
-  button->RequestFocus();
   button->SetModel(nextDayAction);
   ds->Add(button);
 
   ds->Add(new Lum::HSpace(Lum::Space::sizeSmall));
 
+  trend=new Trend();
+  trend->SetModel(this->trend);
+  ds->Add(trend);
+
+  ds->Add(new Lum::HSpace(Lum::Space::sizeSmall));
+
   text=new Lum::TextValue();
   text->SetFlex(true,false);
   text->SetAlignment(Lum::TextValue::right);
-  text->SetMinWidth(Lum::Base::Size::stdCharWidth,5);
+  text->SetMinWidth(Lum::Base::Size::stdCharWidth,6);
   text->SetModel(averageDiff);
   ds->Add(text);
 
@@ -575,7 +583,6 @@ void DataInput::CalcSize()
   table=new Lum::Table();
   table->SetFlex(true,true);
   table->SetMinWidth(Lum::Base::Size::stdCharWidth,30);
-  table->RequestFocus();
   table->SetHeaderModel(headerModel);
   table->SetModel(entries);
   table->SetSelection(entrySelection);
@@ -656,8 +663,6 @@ void DataInput::UpdateEntries()
   entries->Sort(1);
 
   entries->On();
-
-  UpdateSummary();
 }
 
 void DataInput::UpdateFoods()
@@ -703,50 +708,105 @@ void DataInput::UpdateSummary()
                L"/"+
                Lum::Base::NumberToWString(((int)maxValue->Get())/10-current));
 
-  Lum::Base::Calendar d=date->Get();
-  size_t weightLastWeek=0;
-  size_t weightLastWeekCount=0;
-  size_t weightPriorWeek=0;
-  size_t weightPriorWeekCount=0;
-
-  for (size_t i=1; i<=7; i++) {
-    Day *day=data.GetDay(d);
-
+  if (trendType->Get()==trendRefRel) {
     if (day!=NULL) {
-      for (std::list<Entry>::iterator iter=day->entries.begin(); iter!=day->entries.end(); ++iter) {
+      for (std::list<Entry>::reverse_iterator iter=day->entries.rbegin();
+           iter!=day->entries.rend();
+           ++iter) {
         if (iter->item.empty()) {
-          weightLastWeek+=iter->amount;
-          weightLastWeekCount++;
+          if (iter->amount>=refValue->Get()) {
+            averageDiff->Set(AmountToString(iter->amount-refValue->Get())+weightUnit->Get());
+            if (trendDirection->Get()!=1) {
+              trend->Set(Trend::up);
+            }
+            else {
+              trend->Set(Trend::down);
+            }
+          }
+          else {
+            averageDiff->Set(std::wstring(L"-")+AmountToString(refValue->Get()-iter->amount)+weightUnit->Get());
+            if (trendDirection->Get()!=1) {
+              trend->Set(Trend::down);
+            }
+            else {
+              trend->Set(Trend::up);
+            }
+          }
+          return;
         }
       }
     }
-
-    d.AddDays(-1);
-  }
-
-  for (size_t i=1; i<=7; i++) {
-    Day *day=data.GetDay(d);
-
-    if (day!=NULL) {
-      for (std::list<Entry>::iterator iter=day->entries.begin(); iter!=day->entries.end(); ++iter) {
-        if (iter->item.empty()) {
-          weightPriorWeek+=iter->amount;
-          weightPriorWeekCount++;
-        }
-      }
-    }
-
-    d.AddDays(-1);
-  }
-
-  if (weightLastWeekCount==0 || weightPriorWeekCount==0) {
+    trend->Set(Trend::none);
     averageDiff->Set(L"?");
   }
-  else {
-    weightLastWeek=weightLastWeek/weightLastWeekCount;
-    weightPriorWeek=weightPriorWeek/weightPriorWeekCount;
+  else if (trendType->Get()==trendTwoWeeks) {
+    Lum::Base::Calendar d=date->Get();
+    size_t weightLastWeek=0;
+    size_t weightLastWeekCount=0;
+    size_t weightPriorWeek=0;
+    size_t weightPriorWeekCount=0;
 
-    averageDiff->Set(AmountToString(weightLastWeek-weightPriorWeek));
+    for (size_t i=1; i<=7; i++) {
+      Day *day=data.GetDay(d);
+
+      if (day!=NULL) {
+        for (std::list<Entry>::iterator iter=day->entries.begin(); iter!=day->entries.end(); ++iter) {
+          if (iter->item.empty()) {
+            weightLastWeek+=iter->amount;
+            weightLastWeekCount++;
+          }
+        }
+      }
+
+      d.AddDays(-1);
+    }
+
+    for (size_t i=1; i<=7; i++) {
+      Day *day=data.GetDay(d);
+
+      if (day!=NULL) {
+        for (std::list<Entry>::iterator iter=day->entries.begin(); iter!=day->entries.end(); ++iter) {
+          if (iter->item.empty()) {
+            weightPriorWeek+=iter->amount;
+            weightPriorWeekCount++;
+          }
+        }
+      }
+
+      d.AddDays(-1);
+    }
+
+    if (weightLastWeekCount==0 || weightPriorWeekCount==0) {
+      averageDiff->Set(L"?");
+      trend->Set(Trend::none);
+    }
+    else {
+      weightLastWeek=weightLastWeek/weightLastWeekCount;
+      weightPriorWeek=weightPriorWeek/weightPriorWeekCount;
+
+      if (weightPriorWeek<=weightLastWeek) {
+        averageDiff->Set(AmountToString(weightLastWeek-weightPriorWeek)+weightUnit->Get());
+        if (trendDirection->Get()!=1) {
+          trend->Set(Trend::up);
+        }
+        else {
+          trend->Set(Trend::down);
+        }
+      }
+      else {
+        averageDiff->Set(std::wstring(L"-")+AmountToString(weightPriorWeek-weightLastWeek)+weightUnit->Get());
+        if (trendDirection->Get()!=1) {
+          trend->Set(Trend::down);
+        }
+        else {
+          trend->Set(Trend::up);
+        }
+      }
+    }
+  }
+  else {
+    averageDiff->Set(L"???");
+    trend->Set(Trend::none);
   }
 }
 
@@ -959,6 +1019,8 @@ void DataInput::AddWeight()
   entries->Append(new EntriesEntry(entries,data.AddEntry(date->Get(),e)));
 
   entries->Sort(1);
+
+  UpdateSummary();
 }
 
 void DataInput::RemoveItem()
@@ -1126,6 +1188,10 @@ void DataInput::Resync(Lum::Base::Model* model, const Lum::Base::ResyncMsg& msg)
     UpdateFoods();
   }
 
+  else if (model==trendType || model==trendDirection || model==refValue) {
+    UpdateSummary();
+  }
+
   Lum::Component::Resync(model,msg);
 }
 
