@@ -26,9 +26,12 @@
 
 #include "Configuration.h"
 #include "Util.h"
-#include <iostream>
+
 static Diagram::Prefs *prefs=new Diagram::Prefs();
 
+// We resync our self with the date on the first draw
+static bool adjusted=false;
+
 void Diagram::Prefs::Initialize()
 {
   Lum::Scrollable::Prefs::Initialize();
@@ -49,7 +52,11 @@ double exponent(double x)
 
 Diagram::Diagram()
  : warnColor(1.0,0.0,0.0,Lum::OS::display->blackColor),
-   limitColor(1.0,0.75,0.75,Lum::OS::display->blackColor),
+   limitColor(1.0,0.0,0.0,Lum::OS::display->blackColor),
+   valueGoodColor(0.95,0.95,0.95,Lum::OS::display->blackColor),
+   valueGoodFrameColor(0.64,0.64,0.64,Lum::OS::display->blackColor),
+   valueBadColor(1.0,0.82,0.82,Lum::OS::display->blackColor),
+   valueBadFrameColor(0.8,0.58,0.58,Lum::OS::display->blackColor),
    mode(modeFullDay)
 {
   SetPrefs(::prefs);
@@ -63,10 +70,15 @@ Diagram::Diagram()
   AttachModel(bottomValue);
   AttachModel(maxValue);
   AttachModel(minValue);
+
+  // For updating the visible area to the given date
+  AttachModel(date);
 }
 
 Diagram::~Diagram()
 {
+  UnattachModel(date);
+
   UnattachModel(minValue);
   UnattachModel(maxValue);
   UnattachModel(bottomValue);
@@ -138,45 +150,186 @@ size_t Diagram::GetDayWidth() const
   }
 }
 
-void Diagram::Draw(int x, int y, size_t w, size_t h)
+void Diagram::PrintData(Lum::OS::DrawInfo* draw,
+                        size_t dayWidth,
+                        const Lum::Base::Calendar& start,
+                        const Lum::Base::Calendar& end)
 {
-  Lum::Scrollable::Draw(x,y,w,h);
+  int                                                xPos=this->x+((hAdjustment->GetTop()-1)/dayWidth)*
+                                                          dayWidth-(hAdjustment->GetTop()-1);
+  std::map<Lum::Base::Calendar,Day*>::const_iterator iter;
+  bool                                               pointsDrawn=false; // We have drawn at least one point
+  bool                                               pastEnd=false;     // We have beenmoved past the visible end
+  bool                                               finished=false;    // We have drawn one point pastthe visible end and can now finish
+  int                                                px=0,py=0;
+  Lum::Base::Calendar                                day;
+  Value                                              lastWeight=0;
 
-  if (!Intersect(x,y,w,h)) {
-    return;
+  iter=data.data.lower_bound(start);
+  while (iter!=data.data.begin()) {
+    bool hasPoint=false;
+
+    --iter;
+
+    for (std::list<Entry>::const_iterator entry=iter->second->entries.begin();
+         entry!=iter->second->entries.end();
+         ++entry) {
+      if (entry->item.empty()) {
+        hasPoint=true;
+        break;
+      }
+    }
+
+    if (hasPoint) {
+      break;
+    }
   }
 
-  /* --- */
+  while (iter!=data.data.end() && !finished) {
+    int   days;
+    Value value=0;
 
-  DrawBackground(x,y,w,h);
+    //
+    // Detect first weight points to the left and right of the
+    // diagram borders
+    //
 
-  Lum::OS::DrawInfo                                  *draw=GetDrawInfo();
-  Lum::OS::FontRef                                   font;
-  size_t                                             daysCovered=data.GetDaysCovered();
-  Lum::Base::Calendar                                start=data.GetStartDate();
-  Lum::Base::Calendar                                end;
-  Lum::Base::Calendar                                day;
-  std::map<Lum::Base::Calendar,Day*>::const_iterator iter;
-  int                                                xPos;
-  std::wstring                                       tmp;
-  size_t                                             dayWidth;
+    if (iter->first>end) {
+      pastEnd=true;
+    }
 
-  dayWidth=GetDayWidth();
+    days=0;
+    day=start;
+    if (day<=iter->first) {
+      while (day!=iter->first) {
+        day.AddDays(1);
+        days++;
+      }
+    }
+    else {
+      while (day!=iter->first) {
+        day.AddDays(-1);
+        days--;
+      }
+    }
 
-  start.AddDays((hAdjustment->GetTop()-1)/dayWidth);
-  end=start;
-  end.AddDays((width+dayWidth/2)/dayWidth);
+    //
+    // Draw value rectangles
+    //
 
-  hAdjustment->SetDimension(width,std::max(width,daysCovered*dayWidth));
-  hAdjustment->SetStepSize(dayWidth);
+    for (std::list<Entry>::const_iterator entry=iter->second->entries.begin();
+         entry!=iter->second->entries.end();
+         ++entry) {
+      if (!entry->item.empty()) {
+        value+=entry->amount*entry->value;
+      }
+    }
 
-  //
-  // Printing horizontal time scale
-  //
+    if (value>0) {
+      value/=10;
+      if ((size_t)value>maxValue->Get() || (size_t)value<minValue->Get()) {
+        draw->PushForeground(valueBadColor);
+      }
+      else {
+        draw->PushForeground(valueGoodColor);
+      }
+      draw->FillRectangle(xPos+days*dayWidth,this->y+TransformValue(value),
+                          dayWidth,height-TransformValue(value));
+      draw->PopForeground();
 
-  font=Lum::OS::display->GetFont();
+      if ((size_t)value>maxValue->Get() || (size_t)value<minValue->Get()) {
+        draw->PushForeground(valueBadFrameColor);
+      }
+      else {
+        draw->PushForeground(valueGoodFrameColor);
+      }
+      draw->DrawRectangle(xPos+days*dayWidth,this->y+TransformValue(value),
+                          dayWidth,height-TransformValue(value));
+      draw->PopForeground();
+    }
 
-  draw->PushClip(x,y,w,h);
+    //
+    // Draw weight points
+    //
+
+    for (std::list<Entry>::const_iterator entry=iter->second->entries.begin();
+         entry!=iter->second->entries.end();
+         ++entry) {
+      if (entry->item.empty()) {
+        // Draw weight points and connect points by lines
+        if (entry->amount>maxWeight->Get() || entry->amount<minWeight->Get()) {
+          draw->PushForeground(warnColor);
+        }
+        else {
+          draw->PushForeground(Lum::OS::Display::blackColor);
+        }
+
+        int cx,cy;
+
+        cx=xPos+days*dayWidth+(entry->time.GetHour()*dayWidth)/24;
+        cy=this->y+TransformWeight(entry->amount);
+
+        draw->FillArc(cx-3,
+                      cy-3,
+                      6,6,
+                      0,360*64);
+        draw->PopForeground();
+
+        if (pointsDrawn) {
+          int lx1, ly1, lx2, ly2;
+
+          // Project points outside the diagram on the axis of the diagram
+          // to avoid overflow errors and similar...
+          lx1=px;
+          ly1=py;
+          lx2=cx;
+          ly2=cy;
+
+          if (lx1<this->x) {
+            lx1=this->x;
+            ly1=(lx1-px)*(cy-py)/(cx-px)+py;
+          }
+
+          if (lx2>this->x+this->width-1) {
+            lx2=this->x+this->width-1;
+            ly2=(lx2-px)*(cy-py)/(cx-px)+py;
+          }
+
+          if(lastWeight<minWeight->Get() || lastWeight>maxWeight->Get() ||
+             entry->amount<minWeight->Get() || entry->amount>maxWeight->Get()) {
+            draw->PushForeground(warnColor);
+          }
+          else {
+            draw->PushForeground(Lum::OS::Display::blackColor);
+          }
+          draw->PushPen(2,Lum::OS::DrawInfo::penRound);
+          draw->DrawLine(lx1,ly1,lx2,ly2);
+          draw->PopPen();
+          draw->PopForeground();
+        }
+
+        pointsDrawn=true;
+        px=cx;
+        py=cy;
+        lastWeight=entry->amount;
+
+        if (pastEnd) {
+          finished=true;
+        }
+      }
+    }
+
+    ++iter;
+  }
+}
+
+void Diagram::PrintHorizontalScale(Lum::OS::DrawInfo* draw,
+                                   size_t dayWidth,
+                                   const Lum::Base::Calendar& start)
+{
+  Lum::OS::FontRef    font=Lum::OS::display->GetFont();
+  std::wstring        tmp;
+  Lum::Base::Calendar day;
 
   draw->PushForeground(Lum::OS::Display::graphScaleColor);
   draw->PushFont(font);
@@ -184,7 +337,7 @@ void Diagram::Draw(int x, int y, size_t w, size_t h)
   draw->PushClip(this->x+weightUnitWidth,this->y,this->width-weightUnitWidth-valueUnitWidth,this->height);
 
   day=start;
-  xPos=this->x+((hAdjustment->GetTop()-1)/dayWidth)*dayWidth-(hAdjustment->GetTop()-1);
+  int xPos=this->x+((hAdjustment->GetTop()-1)/dayWidth)*dayWidth-(hAdjustment->GetTop()-1);
   while (xPos<this->x+(int)width) {
 
     switch (mode) {
@@ -226,8 +379,90 @@ void Diagram::Draw(int x, int y, size_t w, size_t h)
 
   draw->PopFont();
   draw->PopForeground();
+}
 
-  // Weight limits
+/**
+  Printing vertial value scale
+ */
+void Diagram::PrintVerticalScale(Lum::OS::DrawInfo* draw)
+{
+  Lum::OS::FontRef font=Lum::OS::display->GetFont();
+
+  draw->PushForeground(Lum::OS::Display::graphScaleColor);
+  draw->PushFont(font);
+
+  size_t step;
+  size_t effFontHeight;
+
+  if (font->height/4>2) {
+    effFontHeight=font->height+(font->height/4);
+  }
+  else {
+  effFontHeight=font->height+2;
+  }
+
+  // Weight on left side
+
+  if ((TransformWeight(bottomWeight->Get())-TransformWeight(bottomWeight->Get()+1))>effFontHeight) {
+    step=1;
+  }
+  else if ((TransformWeight(bottomWeight->Get())-TransformWeight(bottomWeight->Get()+5))>effFontHeight) {
+    step=5;
+  }
+  else if ((TransformWeight(bottomWeight->Get())-TransformWeight(bottomWeight->Get()+10))>effFontHeight) {
+    step=10;
+  }
+  else if ((TransformWeight(bottomWeight->Get())-TransformWeight(bottomWeight->Get()+50))>effFontHeight) {
+    step=50;
+  }
+  else if ((TransformWeight(bottomWeight->Get())-TransformWeight(bottomWeight->Get()+100))>effFontHeight) {
+    step=100;
+  }
+  else {
+    step=500;
+  }
+
+  for (size_t i=(bottomWeight->Get()/step)*step; i<=topWeight->Get(); i+=step) {
+    std::wstring tmp=AmountToString(i);
+    draw->DrawString(this->x+weightUnitWidth-font->StringWidth(tmp),
+                     TransformWeight(i)-font->height/2+this->y+font->ascent,
+                     tmp);
+  }
+
+  // Values on right side
+
+  if ((TransformValue(bottomValue->Get())-TransformValue(bottomValue->Get()+1))>effFontHeight) {
+    step=1;
+  }
+  else if ((TransformValue(bottomValue->Get())-TransformValue(bottomValue->Get()+5))>effFontHeight) {
+    step=5;
+  }
+  else if ((TransformValue(bottomValue->Get())-TransformValue(bottomValue->Get()+10))>effFontHeight) {
+    step=10;
+  }
+  else if ((TransformValue(bottomValue->Get())-TransformValue(bottomValue->Get()+50))>effFontHeight) {
+    step=50;
+  }
+  else if ((TransformValue(bottomValue->Get())-TransformValue(bottomValue->Get()+100))>effFontHeight) {
+    step=100;
+  }
+  else {
+    step=500;
+  }
+
+  for (size_t i=(bottomValue->Get()/step)*step; i<=topValue->Get(); i+=step) {
+    std::wstring tmp=AmountToString(i);
+    draw->DrawString(this->x+this->width-1-font->StringWidth(tmp),
+                     TransformValue(i)-font->height/2+this->y+font->ascent,
+                     tmp);
+  }
+
+  draw->PopFont();
+  draw->PopForeground();
+}
+
+void Diagram::PrintLimits(Lum::OS::DrawInfo* draw)
+{
   draw->PushForeground(limitColor);
   draw->DrawLine(this->x,this->y+TransformWeight(maxWeight->Get()),
                  this->x+weightUnitWidth-1,this->y+TransformWeight(maxWeight->Get()));
@@ -242,202 +477,64 @@ void Diagram::Draw(int x, int y, size_t w, size_t h)
   draw->DrawLine(this->x+width-1-valueUnitWidth,this->y+TransformValue(minValue->Get()),
                  this->x+width-1,this->y+TransformValue(minValue->Get()));
   draw->PopForeground();
+}
 
-  //
-  // Printing data
-  //
-
-  xPos=this->x+((hAdjustment->GetTop()-1)/dayWidth)*dayWidth-(hAdjustment->GetTop()-1);
-
-  bool pointsDrawn=false; // We have drawn at least one point
-  bool pastEnd=false;     // We have beenmoved past the visible end
-  bool finished=false;    // We have drawn one point pastthe visible end and can now finish
-  int px=0,py=0;
-
-  iter=data.data.lower_bound(start);
-  while (iter!=data.data.begin()) {
-    bool hasPoint=false;
-
-    --iter;
-
-    for (std::list<Entry>::const_iterator entry=iter->second->entries.begin();
-         entry!=iter->second->entries.end();
-         ++entry) {
-      if (entry->item.empty()) {
-        hasPoint=true;
-        break;
-      }
-    }
-
-    if (hasPoint) {
-      break;
-    }
+void Diagram::SyncGraphWithDate()
+{
+  if (!date.Valid()) {
+    return;
   }
 
-  while (iter!=data.data.end() && !finished) {
-    int days;
-    size_t value=0;
-
-    if (iter->first>end) {
-      pastEnd=true;
-    }
-
-    days=0;
-    day=start;
-    if (day<=iter->first) {
-      while (day!=iter->first) {
-        day.AddDays(1);
-        days++;
-      }
-    }
-    else {
-      while (day!=iter->first) {
-        day.AddDays(-1);
-        days--;
-      }
-    }
-
-    for (std::list<Entry>::const_iterator entry=iter->second->entries.begin();
-         entry!=iter->second->entries.end();
-         ++entry) {
-      if (entry->item.empty()) {
-        if (entry->amount>maxWeight->Get() || entry->amount<minWeight->Get()) {
-          draw->PushForeground(warnColor);
-        }
-        else {
-          draw->PushForeground(Lum::OS::Display::blackColor);
-        }
-
-        int cx,cy;
-
-        cx=xPos+days*dayWidth+(entry->time.GetHour()*dayWidth)/24;
-        cy=this->y+TransformWeight(entry->amount);
-
-        draw->FillArc(cx-3,
-                      cy-3,
-                      6,6,
-                      0,360*64);
-        draw->PopForeground();
-
-        if (pointsDrawn) {
-          int lx1, ly1, lx2, ly2;
-
-          // Project points outside the diagram on the axis of the diagram
-          // to avoid overflow errors and similar...
-          lx1=px;
-          ly1=py;
-          lx2=cx;
-          ly2=cy;
-
-          if (lx1<this->x) {
-            lx1=this->x;
-            ly1=(lx1-px)*(cy-py)/(cx-px)+py;
-          }
-
-          if (lx2>this->x+this->width-1) {
-            lx2=this->x+this->width-1;
-            ly2=(lx2-px)*(cy-py)/(cx-px)+py;
-          }
-
-          draw->PushPen(2,Lum::OS::DrawInfo::penRound);
-          draw->DrawLine(lx1,ly1,lx2,ly2);
-          draw->PopPen();
-        }
-
-        pointsDrawn=true;
-        px=cx;
-        py=cy;
-
-        if (pastEnd) {
-          finished=true;
-        }
-      }
-      else {
-        value+=entry->amount*entry->value;
-      }
-    }
-
-    if (value!=0) {
-      value/=10;
-      if (value>maxValue->Get() || value<minValue->Get()) {
-        draw->PushForeground(warnColor);
-      }
-      else {
-        draw->PushForeground(Lum::OS::Display::blackColor);
-      }
-      draw->DrawLine(xPos+days*dayWidth,this->y+TransformValue(value),
-                     xPos+(days+1)*dayWidth-1,this->y+TransformValue(value));
-      draw->PopForeground();
-    }
-
-
-    ++iter;
+  if (date->Get()<data.GetStartDate()) {
+    GetHAdjustment()->ShowFirstPage();
   }
-
-  //
-  // Printing vertial weight scale
-  //
-
-  draw->PushClip(this->x,this->y,this->width,this->height);
-  draw->PushForeground(Lum::OS::Display::graphScaleColor);
-  draw->PushFont(font);
-
-  size_t step;
-
-  if ((TransformWeight(bottomWeight->Get())-TransformWeight(bottomWeight->Get()+1))>font->height) {
-    step=1;
-  }
-  else if ((TransformWeight(bottomWeight->Get())-TransformWeight(bottomWeight->Get()+5))>font->height) {
-    step=5;
-  }
-  else if ((TransformWeight(bottomWeight->Get())-TransformWeight(bottomWeight->Get()+10))>font->height) {
-    step=10;
-  }
-  else if ((TransformWeight(bottomWeight->Get())-TransformWeight(bottomWeight->Get()+50))>font->height) {
-    step=50;
+  else if (date->Get()>data.GetEndDate()) {
+    GetHAdjustment()->ShowLastPage();
   }
   else {
-    step=100;
+    GetHAdjustment()->CenterOn((date->Get()-data.GetStartDate())*GetDayWidth()+GetDayWidth()/2+1);
+  }
+}
+
+void Diagram::Draw(int x, int y, size_t w, size_t h)
+{
+  Lum::Scrollable::Draw(x,y,w,h);
+
+  if (!Intersect(x,y,w,h)) {
+    return;
   }
 
-  for (size_t i=(bottomWeight->Get()/step)*step; i<=topWeight->Get(); i+=step) {
-    tmp=AmountToString(i);
-    draw->DrawString(this->x+weightUnitWidth-font->StringWidth(tmp),
-                     TransformWeight(i)-font->height/2+this->y+font->ascent,
-                     tmp);
+  size_t dayWidth=GetDayWidth();
+
+  hAdjustment->SetDimension(width,std::max(width,data.GetDaysCovered()*dayWidth));
+  hAdjustment->SetStepSize(dayWidth);
+
+  if (!adjusted) {
+    SyncGraphWithDate();
+    adjusted=true;
   }
 
-  //
-  // Printing vertial value scale
-  //
+  /* --- */
 
-  if ((TransformValue(bottomValue->Get())-TransformValue(bottomValue->Get()+1))>font->height) {
-    step=1;
-  }
-  else if ((TransformValue(bottomValue->Get())-TransformValue(bottomValue->Get()+5))>font->height) {
-    step=5;
-  }
-  else if ((TransformValue(bottomValue->Get())-TransformValue(bottomValue->Get()+10))>font->height) {
-    step=10;
-  }
-  else if ((TransformValue(bottomValue->Get())-TransformValue(bottomValue->Get()+50))>font->height) {
-    step=50;
-  }
-  else {
-    step=100;
-  }
+  DrawBackground(x,y,w,h);
 
-  for (size_t i=(bottomValue->Get()/step)*step; i<=topValue->Get(); i+=step) {
-    tmp=AmountToString(i);
-    draw->DrawString(this->x+this->width-1-font->StringWidth(tmp),
-                     TransformValue(i)-font->height/2+this->y+font->ascent,
-                     tmp);
-  }
+  Lum::OS::DrawInfo   *draw=GetDrawInfo();
+  Lum::Base::Calendar start=data.GetStartDate();
+  Lum::Base::Calendar end;
+
+  start.AddDays((hAdjustment->GetTop()-1)/dayWidth);
+  end=start;
+  end.AddDays((width+dayWidth/2)/dayWidth);
+
+  draw->PushClip(this->x,this->y,width,height);
+  draw->PushClip(x,y,w,h);
+
+  PrintData(draw,dayWidth,start,end);
+  PrintHorizontalScale(draw,dayWidth,start);
+  PrintVerticalScale(draw);
+  PrintLimits(draw);
 
-  draw->PopFont();
-  draw->PopForeground();
   draw->PopClip();
-
   draw->PopClip();
 }
 
@@ -524,6 +621,9 @@ void Diagram::Resync(Lum::Base::Model* model, const Lum::Base::ResyncMsg& msg)
            model==maxValue) {
     Redraw();
   }
+  else if (model==date) {
+    SyncGraphWithDate();
+  }
 
   Scrollable::Resync(model,msg);
 }
